# Copyright 2021 Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

from datetime import datetime
from random import randint

from odoo import _, api, fields, models


class SaleProject(models.Model):
    _name = "sale.project"
    _description = "Sale Project"
    _inherit = ["mail.thread", "mail.activity.mixin"]

    def _get_default_admin_state_id(self):
        return (
            self.env["sale.project.admin.state"]
            .search(
                [("fold", "=", False), ("is_closed", "=", False)],
                order="sequence",
                limit=1,
            )
            .id
        )

    def _get_default_saison_id(self):
        saison_id = self.env["sale.project.saison"].search(
            [
                ("start_date", "<", datetime.today().date()),
                ("end_date", ">=", datetime.today().date()),
            ],
            limit=1,
        )
        if saison_id:
            return saison_id.id

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    sale_order_id = fields.Many2one(
        comodel_name="sale.order",
        string="Devis/Commande",
        required=True,
        ondelete="cascade",
    )
    # Related fields from sale order
    state = fields.Selection(
        related="sale_order_id.state", store=True, string="État de la commande"
    )
    partner_id = fields.Many2one(
        "res.partner", related="sale_order_id.partner_id", store=True, string="Client"
    )
    sale_tag_ids = fields.Many2many(
        related="sale_order_id.tag_ids", string="Étiquettes"
    )
    sale_name = fields.Char(related="sale_order_id.name", string="Réf devis")
    sale_user_id = fields.Many2one(related="sale_order_id.user_id", string="Vendeur")
    sale_amount = fields.Monetary(
        related="sale_order_id.amount_total", string="Montant total devis"
    )
    sale_invoice_status = fields.Selection(
        related="sale_order_id.invoice_status", string="État de la facture"
    )
    currency_id = fields.Many2one(
        related="sale_order_id.currency_id",
        depends=["sale_order_id.currency_id"],
        store=True,
        string="Monnaie",
        readonly=True,
    )

    # Related fields from partner
    partner_city = fields.Char(related="partner_id.city", string="Ville Client")
    partner_phone = fields.Char(related="partner_id.phone", string="Tél Client")
    partner_email = fields.Char(related="partner_id.email", string="Email Client")
    partner_geo_sector_id = fields.Many2one(
        related="partner_id.geo_sector_id", string="Secteur Client"
    )

    name = fields.Char("Nom", default=lambda self: _("Nouveau projet"))

    user_id = fields.Many2one(
        "res.users",
        string="Référent",
        default=lambda self: self.env.user,
        readonly=True,
        states={"draft": [("readonly", False)], "sent": [("readonly", False)]},
    )

    admin_state_id = fields.Many2one(
        comodel_name="sale.project.admin.state",
        string="Étape du projet",
        copy=False,
        ondelete="restrict",
        index=True,
        tracking=1,
        default=_get_default_admin_state_id,
    )
    sale_project_suivi_ids = fields.Many2many(
        comodel_name="sale.project.suivi",
        string="Suivi étape",
        tracking=2,
    )
    is_sale_project_suivi_empty = fields.Boolean(
        string="Pas de suivi pour cette étape",
        compute="_compute_is_sale_project_suivi_empty",
        required=False,
    )
    sale_project_advancement_ids = fields.Many2many(
        comodel_name="sale.project.advancement", string="Avancement", tracking=3
    )

    project_subvention_id = fields.Many2one(
        comodel_name="sale.project.subvention",
        string="Programme",
        ondelete="restrict",
        required=True,
        readonly=True,
        states={"draft": [("readonly", False)], "sent": [("readonly", False)]},
    )
    geo_sector_id = fields.Many2one(
        comodel_name="res.partner.geo.sector",
        string="Secteur géographique",
        readonly=True,
        states={"draft": [("readonly", False)], "sent": [("readonly", False)]},
    )

    saison_id = fields.Many2one(
        "sale.project.saison",
        "Saison",
        default=_get_default_saison_id,
        ondelete="restrict",
    )
    date_visit = fields.Date("Date de visite")

    intervention_ids = fields.One2many(
        comodel_name="sale.intervention",
        inverse_name="project_id",
        string="Interventions",
        readonly=True,
        states={"draft": [("readonly", False)], "sent": [("readonly", False)]},
    )
    # Fields computed from list of interventions
    intervention_counter = fields.Integer(
        compute="_compute_intervention_values",
        string="Nombre d'interventions",
        compute_sudo="True",
    )
    intervention_sequence = fields.Integer(
        string="Séquence d'interventions", default="0"
    )
    intervention_total_length = fields.Float(
        compute="_compute_intervention_values",
        string="Longeur interventions",
        group_operator="sum",
        store=True,
        compute_sudo="True",
    )
    intervention_total_surface = fields.Integer(
        compute="_compute_intervention_values",
        string="Surface interventions",
        group_operator="sum",
        store=True,
        compute_sudo="True",
    )
    intervention_total_plant_qty = fields.Integer(
        compute="_compute_intervention_values",
        string="Nb plants interventions",
        group_operator="sum",
        store=True,
        compute_sudo="True",
    )
    intervention_type_ids = fields.Many2many(
        comodel_name="product.template",
        string="Types d'intervention",
        compute="_compute_intervention_values",
        compute_sudo="True",
    )
    intervention_financial_help_ids = fields.Many2many(
        comodel_name="sale.financial.help",
        string="Aides financières",
        compute="_compute_intervention_values",
        compute_sudo="True",
    )

    sale_order_line_ids = fields.One2many(
        comodel_name="sale.order.line",
        inverse_name="sale_project_id",
        string="Lignes de Devis/Commande",
    )

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    @api.depends("intervention_ids")
    def _compute_intervention_values(self):
        for res in self:
            res.intervention_counter = self.env["sale.intervention"].search_count(
                [("project_id", "=", res.id)]
            )
            res.intervention_total_length = sum(
                res.intervention_ids.mapped("intervention_length")
            )
            res.intervention_total_surface = sum(res.intervention_ids.mapped("surface"))
            res.intervention_total_plant_qty = sum(
                res.intervention_ids.mapped("plant_qty")
            )
            res.intervention_type_ids = res.intervention_ids.mapped(
                "intervention_type_id"
            )
            res.intervention_financial_help_ids = res.intervention_ids.mapped(
                "financial_help_ids"
            )

    def _compute_is_sale_project_suivi_empty(self):
        for r in self:
            sale_project_suivi_count = self.env["sale.project.suivi"].search_count(
                [("sale_project_admin_state_id", "=", self.admin_state_id.id)]
            )
            r.is_sale_project_suivi_empty = (
                True if sale_project_suivi_count == 0 else False
            )

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------
    @api.model_create_multi
    def create(self, vals_list):
        res = super().create(vals_list)
        for rec in res:
            rec.geo_sector_id = rec.sale_order_id.partner_id.geo_sector_id.id
            rec.sale_order_id.partner_id.sale_project_seq += 1
            seq = str(rec.sale_order_id.partner_id.sale_project_seq).zfill(2)
            if rec.saison_id:
                saison_name = "-" + rec.saison_id.name
            else:
                saison_name = ""
            rec.name = rec.sale_order_id.partner_id.ref + saison_name + "-" + seq

    def unlink(self):
        self.sale_order_line_ids.unlink()
        res = super().unlink()
        return res

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def action_interventions(self):
        tree_view = self.env.ref("ap_sale_project.sale_intervention_tree").id
        if self.sale_order_id.state == "draft":
            create = True
        else:
            create = False
        return {
            "type": "ir.actions.act_window",
            "name": "Interventions",
            "res_model": "sale.intervention",
            "view_type": "form",
            "views": [
                [tree_view, "tree"],
                [False, "form"],
                [False, "graph"],
                [False, "pivot"],
            ],
            "domain": [("project_id", "=", self.id)],
            "context": {
                "create": create,
                "default_project_id": self.id,
                "search_default_group_by_intervention_type": True,
            },
        }

    def action_project_form(self):
        return {
            "type": "ir.actions.act_window",
            "res_model": "sale.project",
            "view_mode": "form",
            "res_id": self.id,
        }

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
    def update_order_lines(self):
        # group by intervention_type for current project
        lines = self.env["sale.intervention"].read_group(
            (("project_id", "=", self.id),),
            ("price", "intervention_length", "plant_qty"),
            "intervention_type_id",
            lazy=False,
        )
        for line in lines:
            product_tmpl_id = line["intervention_type_id"][0]
            product_tmpl = self.env["product.template"].browse(product_tmpl_id)
            quantity = (
                line["intervention_length"]
                if product_tmpl.uom_name == "m"
                else line["plant_qty"]
            )
            data = {}
            if quantity != 0:
                data = {
                    "price_unit": line["price"] / quantity,
                    "product_uom_qty": quantity,
                }
                existing_lines = self.sale_order_id.order_line.filtered(
                    lambda r: r.product_id.id == product_tmpl.product_variant_id.id
                    and r.sale_project_id.id == self.id
                )
                if existing_lines:
                    # Update these lines
                    existing_lines.write(data)
                else:
                    # Add missing fields and create new lines
                    data.update(
                        {
                            "order_id": self.sale_order_id.id,
                            "name": "Projet "
                            + self.name
                            + " - "
                            + product_tmpl.product_variant_id.get_product_multiline_description_sale(),
                            "product_id": product_tmpl.product_variant_id.id,
                            "sale_project_id": self.id,
                        }
                    )
                    self.env["sale.order.line"].with_context(
                        default_project_id=False
                    ).create(data)
        self.sale_order_id.clean_sale_order()


class SaleFinancialHelp(models.Model):
    _name = "sale.financial.help"
    _description = "Aide financière"
    _order = "name"

    name = fields.Char("Aide financière", required=True)


class SaleProjectSuivi(models.Model):
    _name = "sale.project.suivi"
    _description = "Suivi étape"
    _order = "sale_project_admin_state_seq, name"

    sale_project_admin_state_id = fields.Many2one(
        string="Étape", comodel_name="sale.project.admin.state", required=True
    )
    sale_project_admin_state_seq = fields.Integer(
        related="sale_project_admin_state_id.sequence", store="True"
    )
    name = fields.Char(string="Suivi étape", required=True)


class SaleProjectAdvancement(models.Model):
    _name = "sale.project.advancement"
    _description = "Avancement"
    _order = "name"

    def _get_default_color(self):
        return randint(1, 11)

    name = fields.Char(string="Avancement", required=True)
    color = fields.Integer(string="Color Index", default=_get_default_color)


class SaleProjectSaison(models.Model):
    _name = "sale.project.saison"
    _description = "Saison"
    _order = "start_date"

    name = fields.Char(string="Saison", required=True)
    start_date = fields.Date("Date de début", required=True)
    end_date = fields.Date("Date de fin", required=True)


class SaleProjectAdminState(models.Model):
    _name = "sale.project.admin.state"
    _description = "Étapes du projet"
    _order = "sequence, id"

    name = fields.Char(string="Avancement", required=True)
    sequence = fields.Integer(default=1)
    is_closed = fields.Boolean(string="Status clos")
    fold = fields.Boolean(string="Plié")
    sale_project_suivi_ids = fields.One2many(
        comodel_name="sale.project.suivi",
        inverse_name="sale_project_admin_state_id",
        string="Suivi des étapes",
    )

    @api.model
    def create(self, vals):
        seq = self.env["ir.sequence"].next_by_code("sale.project.admin.state") or 0
        vals["sequence"] = seq
        return super(SaleProjectAdminState, self).create(vals)
